// 大気圧センサLPS25HのFIFO(I2C接続)のテスト
#include <Wire.h>
#include "skLPSxxI2C.h"

#define FIFO_CTRL_ADRS   0x2E      // FIFOの制御レジスタアドレス
#define FIFO_STATUS_ADRS 0x2F      // FIFOの状態レジスタアドレス

#define LPS25H_ADRS      0x5C      // LPS25HのI2Cアドレス(SDO/SAO is LOW)

// FIFOステータスの構造体
union {
     char sts ;
     struct {
          unsigned WTM_FIFO     :1;
          unsigned FULL_FIFO    :1;
          unsigned EMPTY_FIFO   :1;
          unsigned DIFF_POINT   :5;
     } ;
} fifo ;
unsigned char data[4] ;

skLPSxxx LPS(LPS25H,LPS25H_ADRS) ; // デバイスはLPS25Hを指定

void setup()
{
     int ans ;

     // シリアルモニターの設定
     Serial.begin(9600) ;
     // Ｉ２Ｃの初期化
     Wire.begin() ;                     // マスターとする

     delay(3000) ;                      // 3Sしたら開始
     // 気圧センサ(LPS25H)の初期化を行う
     ans = LPS.PressureInit() ;
     if (ans == 0) {
          // FIFOの設定を行う
          data[0] = 0x40 ; // FIFO ON
          ans = LPS.PressureSend(CTRL_REG2_ADRS,&data[0],1) ;
          data[0] = 0xC7 ; // FIFO_MEAN MODE ･ 8 samples
          ans = LPS.PressureSend(FIFO_CTRL_ADRS,&data[0],1) ;
          Serial.println("Init of LPS25H normal") ;
     } else {
          Serial.print("Init of LPS25H abnormal ans=") ;
          Serial.println(ans) ;
     }
     delay(1000) ;
}
void loop()
{
     int ans ;
     char buf[64] ;
     unsigned char dt[16] ;

     // FIFO_STATUSの読み込み
     ans = LPS.PressureReceive(FIFO_STATUS_ADRS,&data[0],1) ;
     fifo.sts = data[0] ;
     sprintf(buf,"WTM(%d) FULL(%d) EMPTY(%d) data level(%d)  ",
             fifo.WTM_FIFO,fifo.FULL_FIFO,fifo.EMPTY_FIFO,fifo.DIFF_POINT) ;
     Serial.print(buf) ;

     // 圧力・温度読み込み
     LPS.PressureReceive(OUT_DATA_ADRS,&dt[0],3) ;
     LPS.PressureReceive(0x2B,&dt[3],2) ;
     Press = ((unsigned long)dt[2]<<16) + ((unsigned long)dt[1]<<8) + dt[0] ;
     Press = Press / 4096.0 ;      // hPa
     Temp = (int)(1 + ~((unsigned int)dt[4]<<8 | dt[3])) * -1 ;
     Temp = 42.5 + (Temp / 480.0) ;// ℃
     Serial.print(Press) ;              // 気圧値の表示を行う
     Serial.print(" hPa  ") ;
     Serial.print(Temp) ;               // 温度の表示を行う
     Serial.println(" 'C") ;

     delay(1000) ;                      // １秒後に繰り返す
}
